<?php declare(strict_types=1);
namespace DiviManagerPro\Action;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Action\AbstractAction;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\Service\Platform as PlatformService;

/**
 * Because Clone is php keyword use name CloneAction
 *
 * Perform Clone action
 */
class CloneAction extends AbstractAction
{
    const ACTION_CLONE = 'clonesite';


    public function __construct()
    {
        /**
         * Setup the supported commands
         * Might be configurable?
         */
        $this->commands = [
            'dbexport' => [
            ],
            'rsync' => [
            ],
        ];
    }

    public function setPlatformService(PlatformService $sm)
    {
        $this->platformService = $sm;
    }

    public function setProfileService(ProfileService $pm)
    {
        $this->profileService = $pm;
    }

    public function registerActions($worker)
    {
        $worker->actions[self::ACTION_CLONE] = [$this, 'cloneSite'];
    }

    /**
     * Clone a site
     *  - create the new site
     *  - migrate from the source site to the new site
     */
    public function cloneSite( $site, $args )
    {
        error_log(
            __CLASS__.__FUNCTION__ .
            print_r(['site' => $site, 'args' => $args], true)
        );
        $profile = $this->profileService->getProfile($site->getProfileID());
        $platform = $this->platformService->platform($profile->getPlatformID());

        if (null === $platform) {
            error_log( __CLASS__.__FUNCTION__ . ': No platform for site, skipping clone');
            // no platform, skip the install operation
            return $site;
        }

        $appName = $this->siteService->justDomain($site->getDomain());
        // cloning from a profile or a source site?
        if (isset($args['source'])) {
            // given a source to clone from
            $source = $sm->getSite($args['source']);
            if (null === $source) {
                error_log( __CLASS__.__FUNCTION__ . ': Source does not match an existing site. ');
            } else {
                $installID = $platform->cloneInstall($source->getInstallID(), $appName);
            }
        } else {
            // no source, use the profile
            $installID = $platform->cloneInstall($profile->getSource(), $appName);
        }
        $site->setInstallID($installID);

        $this->siteService->updateSite($site);

        // add update ssl to queue, hopefully the delay will be enough?
        $this->queueService->addEventForSiteToQueue(self::ACTION_SSL, $site,);

        return $site;
    }

    /**
     * Migrate a site
     *
     * Copy all the files and database content from one site into another site
     * This is useful for setting up copies for development
     * Also for cloning
     */
    public function migrateSite($srcSite, $dstSite)
    {
        //
    }
}
