<?php declare(strict_types=1);
namespace DiviManagerPro\Action;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Action\AbstractAction;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\Service\Platform as PlatformService;

/**
 * Perform Core action
 */
class Core extends AbstractAction
{
	const ACTION = 'coreupdate';

    public function setPlatformService(PlatformService $sm)
    {
        $this->platformService = $sm;
    }

    public function setProfileService(ProfileService $pm)
    {
        $this->profileService = $pm;
    }

    public function registerActions($worker)
    {
        $worker->actions[self::ACTION] = [$this, 'update'];
    }

    public function update( $site, $args )
    {
        if (null === $site->getInstallID()) {
            throw new \Exception('Site is missing platform install id: ' . $site->getId());
        }

        $command = '/opt/wordpress/bin/wpcli.sh';
        $params = '%user %app %operation';
        $operation =  'core update';

        $platform = $this->platformForSite($site);
        $paths = $platform->paths($site->getInstallID());

        $args = [
            '%user' => $paths[0],
            '%app' => $paths[1],
            '%operation' => $operation
        ];

        // put the parameters and arguments together
        $shell = $this->buildShell($command, $params, $args);

        // run the shell
        return $this->doShell($shell);
    }


    public function platformForSite ( $site )
    {
        $profile = $this->profileService->getProfile($site->getProfileID());
        if (null !== $profile) {
            $platform = $this->platformService->platform($profile->getPlatformID());
            if (null !== $platform) {
                return $platform;
            }
        }

        //Helper::adminMessage('No platform found for this site.', Helper::ERROR);
        return null;
        //throw new \Exception('Missing platform to site: ' . $site->getId());
    }
}
