<?php declare(strict_types=1);
namespace DiviManagerPro\Api;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManagerPro\Service\Server as ServerService;

/**
 * Api for managing servers
 */
class Server
{
    /**
     * @var serverService
     */
    protected $serverService;

    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    /**
     * Return the servers
     */
    public function getList()
    {
        $servers = $this->serverService->getRecords();
        $data = [];
        foreach ($servers as $server) {
            $data[] = $server->toArray();
        }
        return new \WP_REST_Response($data);
    }

    /**
     * Add a new server
     */
    public function create($request)
    {
        $params = $request->get_params();
        $server = $this->serverService->newModel();
        $server->setName($params['name']);
        $server->setIp($params['ip']);
        $server->setIpv6($params['ipv6']);
        $server->setServerID($params['serverID']);
        $server = $this->serverService->createServer($server);
        return new \WP_REST_Response($server->toArray());
    }

    /**
     * Delete an existing server
     */
    public function deleteServer($request)
    {
        $params = $request->get_params();
        $server = $this->serverService->getRecord($params['id']);

        if (null === $server) {
            return new \WP_REST_Response(false);
        }

        $this->serverService->deleteServer($server);
        return new \WP_REST_Response(true);
    }

    public function activate($request)
    {
        // activate the plugin
        include_once ABSPATH . 'wp-admin/includes/plugin.php'; // for activate_plugin
        $result = \activate_plugins( $_POST['slug'] );
        return new \WP_REST_Response($result);
    }

    public function deactivate($request)
    {
        include_once ABSPATH . 'wp-admin/includes/plugin.php'; // for deactivate_plugin
        $result = \deactivate_plugins( $_POST['slug'] );
        return new \WP_REST_Response($result);
    }

    /**
     * Upgrade a plugin
     * 
     * see wp-admin/includes/ajax-actions.php
     */
    public function upgrade($request)
    {
        // setup
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/plugin-install.php'; // For plugins_api()
        require_once ABSPATH . 'wp-admin/includes/update.php';
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include_once ABSPATH . 'wp-admin/includes/class-plugin-upgrader.php';
        //include_once ABSPATH . 'wp-admin/includes/class-class-wp-ajax-upgrader-skin.php';
        // this is for aio-seo...
        if (class_exists('AIOSEO\Plugin\Pro\Admin\Updates')) {
            $aioseoUpdate = new \AIOSEO\Plugin\Pro\Admin\Updates([
                        'pluginSlug' => 'all-in-one-seo-pack-pro',
                        'pluginPath' => plugin_basename( AIOSEO_FILE ),
                        'version'    => AIOSEO_VERSION,
                        'key'        => aioseo()->options->general->licenseKey
            ]);
            add_filter( 'pre_set_site_transient_update_plugins', [ $aioseoUpdate, 'updatePluginsFilter' ], 1000 );
        }       

        // clear the plguin and updates cache
        //\delete_site_transient( 'update_plugins' );
        \wp_update_plugins();
        $plugins = get_plugins();
        $updates = get_site_transient( 'update_plugins' );
        //\wp_clean_plugins_cache(true);
        // wp ajax handles nonce checking automatically?
        $skin     = new \WP_Ajax_Upgrader_Skin();
        $upgrader = new \Plugin_Upgrader( $skin );

        $slug = $_POST['slug'];
        $files = $request->get_file_params();
        $status = [
            'update'     => 'plugin',
            'slug'       => $slug,
            'oldVersion' => '',
            'newVersion' => '',
        ];

        // this needs redone?
        // check the slug first
        // check if the plugin can be upgraded normally
        // then, check if the file is being sent?
        if (! empty($files)) {
            $file = $files['archive'];
            $this->addToTransient($slug, $file['tmp_name']);
            $result = $upgrader->upgrade( $slug );
            if (\is_wp_error($result)) {
                return $result;
            }
        } else {
            //$result = $upgrader->upgrade( $slug );
            $result   = $upgrader->bulk_upgrade( [ $slug ] );
            if ( is_wp_error( $skin->result ) ) {
                $status['errorCode']    = $skin->result->get_error_code();
                $status['errorMessage'] = $skin->result->get_error_message();
                wp_send_json_error( $status );
            } elseif ( $skin->get_errors()->has_errors() ) {
                $status['errorMessage'] = $skin->get_error_messages();
                wp_send_json_error( $status );
            } elseif ( is_array( $result ) && ! empty( $result[ $slug ] ) ) {
                // need to activate the plugin as well.
                $activated = activate_plugin( $slug, '' );
                if ( is_wp_error( $activated ) ) {
                    $activated->add_data( [ 'status' => 500 ] );

                    return $activated;
                } else {
                    wp_send_json_success( $status );
                }
            } elseif ( false === $result ) {
                wp_send_json_error( $status );
            }

            $status['errorMessage'] = __( 'Plugin update failed.' );
            wp_send_json_error( $status );
        }

        return new \WP_REST_Response($result);
    }
}
