<?php declare(strict_types=1);
namespace DiviManagerPro\Command;

use DiviManager\Service\Site as SiteService;

/**
 * Clone is a php keyword.  Use CloneCommand instead
 *
 * Trigger a clone of a site
 */
class CloneCommand extends \WP_CLI_Command
{
    /**
     * @var SiteService
     */
    protected $siteService;

    public function setSiteService(SiteService $sm)
    {
        $this->siteService = $sm;
    }

    /**
     * Clone a site
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * <dst_site_id>
     * : The id of the site which is the destination of the clone.
     *
     * [--dry-run]
     * : Just print out the steps, do not do them.  True by default
     *
     *
     * ## EXAMPLES
     *
     *     wp manager clone <src_site_id> <dst_site_id> [--dry-run=<dry-run>]
     *
     * @subcommand clone
     * @subcommand clone fromUser fromApp toUser domain
     */
    public function cloneSite($args, $assoc_args )
    {
        $srcSiteID = $args[0];
        $dstSiteID = $args[1];
        $dryrun = isset($assoc_args['dry-run']) ? $assoc_args['dry-run'] : true;

        $site = $this->siteService->getSite($srcSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Source site not found' );
        }

        $site = $this->siteService->getSite($dstSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Destination site not found' );
        }

        \WP_CLI::log( print_r($srcSiteID, true) );
        \WP_CLI::log( print_r($dstSiteID, true) );
        \WP_CLI::log( print_r($dryrun, true) );

        $action = new CloneAction;
        $action->setDryrun($dryrun);
        $action->cloneSite($args);
        \WP_CLI::success( 'Success' );
    }
}
