<?php declare(strict_types=1);
namespace DiviManagerPro\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Helper;
use DiviManager\Service\Site as SiteService;
use DiviManagerPro\Table\PlatformInstall as PlatformInstall;
use DiviManagerPro\Table\PlatformServer as PlatformServerTable;
use DiviManagerPro\Service\Platform as PlatformService;
use DiviManagerPro\Service\PlatformServer as PlatformServerService;
use DiviManagerPro\Service\Server as ServerService;

/**
 * Display platforms
 */
class Platform
{
    public function __construct()
    {
    }

    public function setPlatformService(PlatformService $pm)
    {
        $this->platformService = $pm;
    }

    public function setPlatformServerService(PlatformServerService $pm)
    {
        $this->platformServerService = $pm;
    }

    public function setSiteService(SiteService $pm)
    {
        $this->siteService = $pm;
    }

    public function setPlatformTable(PlatformInstall $pi)
    {
        $this->platformTable = $pi;
    }

    public function setPlatformServerTable(PlatformServerTable $t)
    {
        $this->platformServerTable = $t;
    }

    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    /**
     * Main page
     */
    public function index ()
    {
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
<?php
        $this->platforms();
?>
	</div>
<?php
    }

    // Add screen options
    public function screen_option ()
    {
        $option = 'per_page';
        $args   = [
            'label'   => __( 'Sites' ),
            'default' => 100,
            'option'  => 'sites_per_page'
        ];
    }

    /**
     * Show registered platforms
     */
    public function platforms ()
    {
        $platforms = $this->platformService->activePlatforms();

        echo <<<EOHTML
<p>Platforms are site hosting services.  DiviManager is capable of interacting with some platforms.  Capabilities vary per platform.  Some include: deploying a new site, changing details of the site on that platform.  Some platforms support cloning or migrating sites.</p>
<p>Some platforms are server agnostic.  For platforms which have individual servers these must be managed as well.</p>
EOHTML;
        if (0 === count($platforms)) {
            echo '<p>No active platforms.  Please install a platform extension for DiviManager.</p>';
            return;
        }

        echo <<<EOHTML
<table>
   <tbody>
       <tr><th>Platform</th><th>Description</th><th>Capabilities</th></tr>
EOHTML;
        foreach ($platforms as $platform) {
            $detailsUrl =  admin_url( 'admin.php?page=divimanager-platform-details&platform=' . $platform->getId() );
            echo <<<EOHTML
       <tr><td><a href="$detailsUrl">{$platform->getName()}</a></td><td>{$platform->getDescription()}</td><td>{$platform->getCapabilities()}</td></tr>
EOHTML;
        }
        echo <<<EOHTML
   </tbody>
</table>
EOHTML;
    }

    /**
     * Show details for selected platform
     */
    public function details()
    {
        $platform_id = $_GET['platform'];
        $platform = $this->platformService->platform($platform_id);
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
<?php
        if (null === $platform) {
            echo '<p>Unregistered platform.</p>';
        } else {
            try {
                $this->platformTable->prepare_items();
            } catch (\Exception $e) {
                Helper::adminMessage('We encountered an error with the sites: ' . $e->getMessage(), Helper::ERROR);
            }

            if (isset($_POST['import']) && 'Import' === $_POST['import']) {
                var_dump($_POST['import']);
                die(__FILE__);
                // we handle the imports, the platform gives the site information
                foreach ($platform->getInstalls() as $install) {
                    if (isset($_POST['install_ids'])) {
                        if (! in_array($install['id'], $_POST['install_ids'])) {
                            continue;
                        }
                    }
                    $domain = $this->siteService->justDomain($install['domain']);
                    $site = $sm->getSiteByDomain($domain);
                    if ($site) {
                        // site already exists
                        // maybe update the order and customer?
                        $site->setInstallID( $install['id'] );
                        $site->setTimestamp(null);
                        $sm->updateSite($site);
                    } else {
                        $site = $sm->newSite();
                        $site->setUrl( $domain );
                        $site->setDomain( $domain );
                        $site->setName( $domain );
                        $site->setInstallID( $install['id'] );
                        $site->setTimestamp(null);
                        $site->setUserID( 0 );
                        $sm->createSite($site);
                    }
                }
            }

            //$serversUrl =  admin_url( 'admin.php?page=divimanager-server&platform=' . $platform->getId() );
            $platform_id = $_GET['platform'];
            $platform = $this->platformService->platform($platform_id);
?>
        <div id="post-body" class="metabox-holder columns-2">
            <div id="post-body-content">
            <p>Platform: <?php echo $platform->getName(); ?></p>
            <p><?php echo $platform->getDescription(); ?></p>
            <p>You can click the checkbox next to individual sites to import just those sites.  Installs show the manager id if they are already recognized, these can be imported to refresh any platform specific data.</p>
                <div class="meta-box-sortables ui-sortable">
                    <form method="POST">
<?php
            try {
                $this->platformTable->display();
            } catch (\Exception $e) {
                Helper::adminMessage('We encountered an error with the sites: ' . $e->getMessage(), Helper::ERROR);
            }
?>
                    </form>
                </div>
            </div>
        </div>
        <span>Total Installs:</span><span><?php echo $this->platformTable->getStats()['total']; ?></span>
        <span>Matched:</span><span><?php echo $this->platformTable->getStats()['matched']; ?></span>
        <span>Found:</span><span><?php echo $this->platformTable->getStats()['found']; ?></span>
        <span>Missing:</span><span><?php echo $this->platformTable->getStats()['missing']; ?></span>
        <br class="clear">
        <p>Domains</p>
<?php
            /*
            $domains = $platform->getDomains();
            foreach ($domains as $domain) {
                if (is_string($domain)) {
                    echo '<span>' . $domain . '</span></br>';
                } else {
                    echo '<span>' . $domain->name . '</span></br>';
                }
            }
             */
?>
<?php
        }
?>
        <br class="clear">
	</div>
<?php
    }

    /**
     * Show servers
     */
    public function servers ($platform)
    {
        $platform_id = $_GET['platform'];
        $platform = $this->platformService->platform($platform_id);
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
<?php
        if (null === $platform) {
            echo '<p>Unregistered platform.</p>';
        } else {
            try {
                $this->platformServerTable->prepare_items();
            } catch (\Exception $e) {
                Helper::adminMessage('We encountered an error with the servers: ' . $e->getMessage(), Helper::ERROR);
            }

            $platform_id = $_GET['platform'];
            $platform = $this->platformService->platform($platform_id);
?>
        <div id="post-body" class="metabox-holder columns-2">
            <div id="post-body-content">
            <p>Platform: <?php echo $platform->getName(); ?></p>
            <p><?php echo $platform->getDescription(); ?></p>
            <p>You can click the checkbox next to individual servers to import just those servers.  Installs show the manager id if they are already recognized, these can be imported to refresh any platform specific data.</p>
                <div class="meta-box-sortables ui-sortable">
                    <form method="POST">
<?php
            try {
                $this->platformServerTable->display();
            } catch (\Exception $e) {
                Helper::adminMessage('We encountered an error with the servers: ' . $e->getMessage(), Helper::ERROR);
            }
?>
                    </form>
                </div>
            </div>
        </div>
        <span>Total Servers:</span><span><?php echo $this->platformServerTable->getStats()['total']; ?></span>
        <span>Matched:</span><span><?php echo $this->platformServerTable->getStats()['matched']; ?></span>
        <span>Found:</span><span><?php echo $this->platformServerTable->getStats()['found']; ?></span>
        <span>Missing:</span><span><?php echo $this->platformServerTable->getStats()['missing']; ?></span>
        <br class="clear">
        <p>Domains</p>
<?php
            /*
            $domains = $platform->getDomains();
            foreach ($domains as $domain) {
                if (is_string($domain)) {
                    echo '<span>' . $domain . '</span></br>';
                } else {
                    echo '<span>' . $domain->name . '</span></br>';
                }
            }
             */
?>
<?php
        }
?>
        <br class="clear">
	</div>
<?php
    }
}
