<?php declare(strict_types=1);
namespace DiviManagerPro\Platform;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

/**
 * Define the expected api for platform classes
 */
interface PlatformInterface
{
    /**
     * Helpers to check if this profile uses clone or does a new install
     *
     * @return bool
     */
    public function doesClone();
    public function doesProvision();

    /**
     * API Wrapper
     *
     * These are the behaviors expected from all platforms
     */

    /**
     * List the sites hosted on the platform
     */
    public function installs();

    /**
     * PRovision a new site on the platform
     *
     * @param String name of the new site, usually a domain name
     */
    public function provision($name);

    /**
     * Clone an existing site on the platform
     *
     * @param var identifier for the source to clone, specific to the platform
     * @param String name of the new install
     */
    public function cloneInstall($sourceId, $name);

    /**
     * Deactivate a site
     *
     * @param var identifier of the site to deactivate, specific to platform
     */
    public function deactivate($id);

    /**
     * Change setting of a site
     *
     * @param var idtentifier of the site to alter, specific to platform
     * @param String setting string to alter
     * @param var value to set
     */
    public function alter($id, $setting, $value);
}
