<?php declare(strict_types=1);
namespace DiviManagerPro\Reporter;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManagerPro\Reporter\Reporter;
use DiviManager\Registry;

// use wordpress's site health class
use WP_Site_Health;

/**
 * Support a new admin menu
 *
 * @see https://codex.wordpress.org/Adding_Administration_Menus
 */
class Admin
{
    /**
     * @var Registry the core class
     */
    protected $registry;

    /**
     * @var String plugin slug, used as base for options
     */
    protected $slug;

    /**
     * @var Option options manager
     */
    protected $option;

    /**
     * @var Reporter the reporter extension main class
     */
    protected $reporter;

    public function __construct()
    {
        $this->slug = 'divimanager-pro';
    }

    public function setOption($option)
    {   
        $this->option = $option;
    }

    public function setReporter(Reporter $r)
    {
        $this->reporter = $r;
    }

    public function setRegistry(Registry $registry)
    {   
        $this->registry = $registry;
    }

    public function setSlug(String $s)
    {
        $this->slug = $s;
    }

	public function init()
    {   
        add_action( 'admin_menu', [$this, 'admin_menu'], 100 );
        add_filter( 'plugin_action_links_divimanager-reporter/divimanager-reporter.php', [$this, 'add_action_links'] );
        add_filter( 'divimanager_mysettings', [$this, 'register_mysettings'], 3 );
        add_filter( 'divimanager_options_validate', [$this, 'plugin_options_validate'] );
    }
    
    public function register_mysettings($settings)
    {
        $sectionName = $this->slug . '_section_reporter';
        $optionName = $this->option->getOptionName();
        $settings[$this->slug . '-reporter'] = [
			$sectionName,
			'Reporter Settings',
			[$this, 'reporter_settings'],
			$optionName,
            'settings' => [
            ]
        ];
        $settings[$this->slug . '-reporter']['settings'][] = [   
            $this->slug . '_field_reporter_enabled',
            'Reporter Enabled',
            [$this, 'plugin_setting_reporter_enabled'],
            $optionName,
            $sectionName,
            [   
                'label_for' => 'divimanager_options[reporter_enabled]'
            ]
        ];
        if ($this->reporter->isReporterEnabled()) {
            $settings[$this->slug . '-reporter']['settings'][] = [
                $this->slug . '_field_report_email',
                'Email To(default)',
                [$this, 'plugin_setting_email'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[report_email]'
                ]
            ];
            $settings[$this->slug . '-reporter']['settings'][] = [
                $this->slug . '_field_report_cc',
                'Cc',
                [$this, 'plugin_setting_cc'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[report_cc]'
                ]
            ];
            $settings[$this->slug . '-reporter']['settings'][] = [
                $this->slug . '_field_report_subject',
                'Email Subject',
                [$this, 'plugin_setting_subject'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[report_subject]'
                ]
            ];
            $settings[$this->slug . '-reporter']['settings'][] = [
                $this->slug . '_field_report_template',
                'Email Template',
                [$this, 'plugin_setting_template'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[report_template]'
                ]
            ];
        }

		return $settings;
    }

    public function add_action_links ($actions)
    {
        $links = [
             '<a href="' . admin_url( 'admin.php?page=divimanager-options' ) . '">Settings</a>',
        ];
        return array_merge($links, $actions);
    }

	public function admin_menu ()
    {
        $mainPage = $this->registry->getService(Page\Main::class);
        $reportPage = $this->registry->getService(Page\Report::class);

        $reportMenuHook = add_submenu_page( 'divimanager-menu', 'Reporter', 'Reporter', 'manage_options', 'divimanager-reporter-main', [$mainPage, 'main'] );
        //$reportMenuHook = add_submenu_page( 'reporter-menu', 'Reporter', 'Reporter', 'manage_options', 'reporter-menu', [$this, 'admin_report']);
        //$mainHook = add_submenu_page( 'divimanager-menu', 'Manager', 'Manager', 'manage_options', 'divimanager-menu', [$mainPage, 'admin_main']);
        //add_submenu_page( 'reporter-menu', 'Report', 'Report', 'manage_options', 'reporter-menu', [$reportPage, 'report']);
        //add_submenu_page( $reportMenuHook, 'Shortcodes', 'Shortcodes', 'manage_options', 'divimanager-reporter-shortcode', [$this, 'admin_shortcode']);
        //$reportHook = add_submenu_page( $reportMenuHook, 'Report', 'Report', 'manage_options', 'reporter-report', [$reportPage, 'report']);
        $reportHook = add_submenu_page( $reportMenuHook, 'Report', 'Report', 'manage_options', 'divimanager-reporter-site', [$reportPage, 'report']);

	}

    /**
     * custom plugin settings
     */
    public function reporter_settings($args)
    {
        echo '<p id="' . esc_attr( $args['id'] ) . '">All the settings which the report aggregator extension will use.  Leave blank to use default values.</p>';
    }

    public function plugin_setting_reporter_enabled($args)
    {
        $enabled = $this->option->getOption('reporter_enabled');
        if (true == $enabled) {
            $checked = 'checked';
        } else {
            $checked = '';
        }
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="checkbox" size="40" value="true" '.$checked.'/>';
    }

    public function plugin_setting_email($args)
    {
        $email = $this->option->getOption('report_email');
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" size="40" value="' . esc_attr ( $email ) . '" />';
    }

    public function plugin_setting_cc($args)
    {
        $cc = $this->option->getOption('report_cc');
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" size="40" value="' . esc_attr ( $cc ) . '" />';
    }

    public function plugin_setting_subject($args)
    {
        $subject = $this->option->getOption('report_subject');
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" size="40" value="' . esc_attr ( $subject ) . '" />';
    }

    public function plugin_setting_template($args)
    {
        $template = $this->option->getOption('report_template');
        echo "<textarea cols='80' rows='40' id='" . esc_attr( $args['label_for'] ) . "' name='" . esc_attr( $args['label_for'] ) . "'>{$template}</textarea>";
    }

    public function plugin_options_validate($input)
    {
        $input['reporter_enabled'] = isset($input['reporter_enabled']) ? true == \sanitize_text_field($input['reporter_enabled']) : false;
        $input['report_email'] = isset($input['report_email']) ? \sanitize_text_field($input['report_email']) : null;
        $input['report_cc'] = isset($input['report_cc']) ? \sanitize_text_field($input['report_cc']) : null;
        $input['report_subject'] = isset($input['report_subject']) ? \sanitize_text_field($input['report_subject']) : null;
        //$input['plugin_template'] = sanitize_textarea_field($_POST['reporter_options']['plugin_template']);
        //$input['plugin_template'] = wp_kses_post($_POST['reporter_options']['plugin_template']);
        $input['report_template'] = isset($input['report_template']) ? trim($input['report_template']) : null;

        return $input;
    }
}
