<?php
namespace DiviManagerPro\Reporter\Manager;

use DiviManagerPro\Model\SiteTemplate as SiteTemplateModel;
use \Rollbar\Rollbar;
use \Rollbar\Payload\Level;

/**
 * Service class for working with site templates for reporter
 *
 * Singleton pattern
 */
class SiteTemplate
{
    /**
     * var SiteTemplate
     */
    static $instance = false;

    /**
     * var String tablename
     */
    protected $tableName = null;

    public function __construct()
    {
        global $wpdb;
        $this->tableName = $wpdb->prefix . 'sl_templates';
    }

    /**
     * Singleton pattern
     */
    public static function getInstance()
    {
        if ( !self::$instance ) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * Setup our cron hook
     */
    public function init()
    {
    }

    public function activation()
    {
        $this->install();
    }

    public function deactivation()
    {
        $timestamp = wp_next_scheduled( self::CRON_DAILY_HOOK );
        wp_unschedule_event( $timestamp, self::CRON_DAILY_HOOK );

        //$this->uninstall();
    }

	/**
     * install db schema
     */
    public function install ()
    {
        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();

        $sql =<<<EOSQL
DROP TABLE IF EXISTS $this->tableName;
CREATE TABLE $this->tableName (
id int NOT NULL AUTO_INCREMENT,
user_id int NOT NULL,
template text null,
PRIMARY KEY (id)
) $charset_collate;
EOSQL;
        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    }

    /**
     * uninstall the schema
     */
    protected function uninstall ()
    {
        global $wpdb;

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( 'DROP TABLE IF EXISTS ' . $this->tableName . ';' );
    }

    public function addQueryVars( $vars ){
        $vars[] = 'site_id';
        return $vars;
    }

    public function newSite ()
    {
        return new SiteModel;
    }

	public function getCount()
	{
		global $wpdb;

		$sql =<<<EOSQL
SELECT COUNT(*) FROM {$this->tableName}
EOSQL;

		return $wpdb->get_var( $sql );
	}

    public function getTemplates ($perPage = 150, $pageNumber = 1, $options = null)
    {
        global $wpdb;
        $start = $perPage * ($pageNumber-1);

        $sql =<<<EOSQL
SELECT {$this->tableName}.* FROM {$this->tableName} 
EOSQL;

        $where = [];
        if (null !== $options) {
            if(isset($options['site']) && ! empty($options['site'])) {
                $where[] = 'site_id = ' . $options['site'];
            }
            if(isset($options['s']) && ! empty($options['s'])) {
                $where[] = 'data like "%' . $options['s'] . '%"';
            }
        }


        if (! empty($where)) {
            $sql .= ' WHERE ' . implode(' AND ', $where);
        }

        if (null !== $options && isset($options['orderby']) && ! empty($options['orderby'])) {
            $orderby = $this->tableName . '.' . $options['orderby'];
            if (isset($options['order'])) {
                $orderby .= ' ' . $options['order'];
            }
        } else {
            $orderby = $this->tableName . '.ID ASC';
        }
        $sql.=<<<EOSQL
 ORDER BY $orderby LIMIT $perPage OFFSET $start
EOSQL;
        $sites = [];
        return $wpdb->get_results( $sql, ARRAY_A );
    }

    public function getTemplate ($id)
    {
         global $wpdb;

        $sql =<<<EOSQL
SELECT {$this->tableName}.* FROM {$this->tableName} WHERE {$this->tableName}.id=$id
EOSQL;

         $site = $wpdb->get_row( $sql, ARRAY_A);
         if (null === $site) {
             return null;
             throw new \RuntimeException('site not found in database:'. $siteID);
         }

         $newTemplate = $this->newTemplate();

         return $newTemplate->fromArray($template);
    }

    public function getTemplatebySite ($siteID)
    {
         global $wpdb;

        $sql =<<<EOSQL
SELECT {$this->tableName}.* FROM {$this->tableName} WHERE {$this->tableName}.site_id="%s"
EOSQL;

         $preparedSql = $wpdb->prepare($sql, $siteID);

         $template = $wpdb->get_row( $preparedSql, ARRAY_A);
         if (null === $template || empty($template)) {
             return null;
         }

         $newTemplate = $this->newTemplate();

         return $newTemplate->fromArray($template);
    }

    public function createTemplate (SiteTemplateModel $model)
    {
        global $wpdb;

        $ok = $wpdb->insert( $this->tableName, [
            'site_id' => $model->getSiteID(),
            'template'  => $model->getTemplate(),
        ]);

        if (false === $ok) {
            throw new \Exception('Failed to create template:' . print_r($model->getId(), true) . ':' . print_r($model, true));
        }

        $site->setId($wpdb->insert_id);

        return $site;
    }

    public function updateTemplate (SiteTemplateModel $model)
    {
        global $wpdb;

        if (is_string($site->getTimestamp())) {
            throw new \Exception('Invalid timestamp for site');
        }
        $result = $wpdb->update( $this->tableName, [
            'site_id' => $site->getSiteID(),
            'template'  => $site->getTemplate(),
        ], ['id' => $site->getId()]);

        if (false === $result) {
            throw new \Exception('Failed to update site:' . print_r($site->getId(), true) . ':' . print_r($site, true));
        }

        return $site;
    }

    public function deleteTemplate (SiteTemplateModel $model)
    {
        global $wpdb;

        $wpdb->delete( $this->tableName, ['id' => $model->getId()]);

        return [];
    }
}
