<?php declare(strict_types=1);
namespace DiviManagerPro\Reporter\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManagerPro\DiviManagerPro;
use DiviManager\Service\Site as SiteService;
use DiviManagerPro\Reporter\Service\Reporter as ReporterService;

/**
 * Show the main setup of the reporter
 */
class Main
{
    /**
     * @var Option options manager
     */
    protected $option;

    /**
     * @var DiviManagerPro the DiviManagerPro extension main class
     */
    protected $divimanagerpro;

    /**
     * @var ReporterService
     */
    protected $reporterService;

    public function __construct()
    {
        // load our default template
        $templateFile = WP_PLUGIN_DIR . '/divimanager-pro/templates/email.phtml';
        $this->template = file_get_contents($templateFile);
    }

    public function setOption($option)
    {   
        $this->option = $option;
    }

    public function setDiviManagerPro(DiviManagerPro $r)
    {
        $this->divimanagerpro = $r;
    }

    public function setReporterService(ReporterService $r)
    {
        $this->reporterService = $r;
    }

    /**
     * Show setup for reporter
     */
    public function main ()
    {
        $to = $this->option->getOption('report_email');
        $cc = $this->option->getOption('report_cc');
        $subject = $this->option->getOption('report_subject');
        $emailBody = $this->option->getOption('report_template');
        $lastRunDate = $this->option->getOption('report_last_run');
        //if (! $lastRunDate instanceof \Datetime) {
            //$timestamp = wp_next_scheduled( DiviManagerPro::CRON_HOOK );
            //$nextReportDate = new \Datetime;
            //$nextReportDate->setTimestamp($timestamp);
            //$lastRunMessage = 'The report has not yet been run.  The report is scheduled to run on: ' . $nextReportDate->format('Y-m-d');

        //} else {
            $lastRunMessage = $lastRunDate;//->format('Y-m-d');
        //}
        $shortcodes = $this->shortcodes();
        echo <<<EOHTML
<p>The report was last run for the month ending:$lastRunMessage.</p>
<p>The report will be sent to: $to</p>
<p>The report will be Cc'd: $cc</p>
<p>For the following, the shortcodes have not been processed.  In the actual emails the shortcodes will be processed.</p>
<p>The default email subject will be: $subject</p>
<p>The following is default email template:</p>
<div style="border: 1px black solid;">$emailBody</div>
<p>To send the report please click the Send button below.</p>
$shortcodes
EOHTML;
    }

    /**
     * Show shortcodes
     */
    public function shortcodes ()
    {
        $codes = $this->reporterService->getContentCodes();
        $html = <<<EOHTML
<h1>Shortcodes</h1>
<p>See the following for all the content available in the report.</p>
<table>
<thead>
<tr><th>Section</th><th>Shortcode</th><th>Description</th></tr>
</thead>
<tbody>
EOHTML;
        foreach ($codes as $section => $details) {
            $html .= sprintf('<tr><td colspan=4 style="border: 1px black solid;">%s</td></tr>', $section);
            foreach ($details['content'] as $code => $description) {
                $html .= sprintf('<tr><td style="border: 1px black solid;">%s</td><td style="border: 1px black solid;">%s</td><td style="border: 1px black solid;">%s</td></tr>', '', $code, $description);
            }
        }


        $html .=<<<EOHTML
</tbody>
</table>
EOHTML;

        return $html;
    }
}
