<?php declare(strict_types=1);
namespace DiviManagerPro\Reporter\Table;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;

use DiviManager\Table\AbstractTable;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\Queue as QueueService;
use DiviManagerPro\Action\Reporter as ReporterAction;
use DiviManager\Helper;

/**
 * Add our extensions to the divimanager site table
 */
class Site
{
    // unique prefix for hook names
    // hdm?  mngr?  hdmngr? 
    public const HOOK_PREFIX = 'hdm';

    public $wpLatest;
    public $stats;
    public $statsTransient;

    /**
     * @var SiteService
     */
    protected $siteService;

    /**
     * @var QueueService
     */
    protected $queueService;

	public function __construct()
    {
        $this->init();
    }

    public function setSiteService(SiteService $s)
    {
        $this->siteService = $s;
    }

    public function setQueueService(QueueService $s)
    {
        $this->queueService = $s;
    }

    public function init()
    {
        //\add_filter (self::HOOK_PREFIX . '_table_columns_init', [$this, 'myColumns'], 1, 1);
        \add_filter (self::HOOK_PREFIX . '_table_actions_init', [$this, 'myActions']);
        \add_filter (self::HOOK_PREFIX . '_table_quickactions_init', [$this, 'myQuickActions'], 10, 2);
    }

	/**
	 * Define the bulk actions this table supports
	 *
	 * @return array
	 */
    public function myActions($actions)
    {
        $actions['bulk-report'] = ['label' => 'Report', 'bulk' => true, 'callback' => [$this, 'action_bulk_report']];
        $actions['report'] = ['label' => 'Report', 'bulk' => false, 'callback' => [$this, 'action_report']];

	    return $actions;
    }

	/**
	 * Define the quick actions this table supports
	 *
	 * @return array
	 */
    public function myQuickActions($actions, $site)
    {
        $site_id = $site->getId();
        $reportUrl = esc_url ( add_query_arg( ['page' => 'divimanager-reporter-site', 'site_id' => $site_id], admin_url('admin.php')));
        $actions['report'] = sprintf( '<a href="%s">Report</a>', $reportUrl);

        return $actions;
    }

    /***
     * Bulk actions
     */
    /**
     * Send report for multiple sites
     */
    public function action_bulk_report($ids)
    {
        foreach ( $ids as $id ) {
            $site = $this->siteService->getSite($id);
            $this->queueService->addEventForSiteToQueue(ReporterAction::ACTION_REPORT, $site);
        }
    }


    /***
     * Single site actions
     */
    /**
     * Refresh one sites
     */
    public function action_report($args)
    {
        Helper::adminMessage('Performing action: ' . ReporterAction::ACTION_REPORT, Helper::SUCCESS);
        $site = $this->siteService->getSite($args['site_id']);
        $this->queueService->addEventForSiteToQueue(ReporterAction::ACTION_REPORT, $site);
    }
}
