<?php declare(strict_types=1);
namespace DiviManagerPro\Serverpilot;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;

/**
 * Support configuration and admin features for the Serverpilot platform
 */
class Admin
{
    /**
     * @var Registry the core class
     */
    protected $registry;

    /**
     * @var String plugin slug, used as base for options
     */
    protected $slug;

    /**
     * @var Option options manager
     */
    protected $option;

    /**
     * @var Serverpilot the serverpilot extension main class
     */
    protected $serverpilot;

    public function __construct()
    {
        $this->slug = 'divimanager-pro';
    }

    public function setOption($option)
    {   
        $this->option = $option;
    }

    public function setServerpilot(Serverpilot $s)
    {
        $this->serverpilot = $s;
    }

    public function setRegistry(Registry $registry)
    {   
        $this->registry = $registry;
    }

    public function setSlug(String $s)
    {
        $this->slug = $s;
    }

	public function init()
    {   
        add_action( 'admin_menu', [$this, 'admin_menu'] );
        add_filter( 'divimanager_mysettings', [$this, 'register_mysettings'], 4 );
        add_filter( 'divimanager_options_validate', [$this, 'plugin_options_validate'] );
    }
    
    public function register_mysettings($settings)
    {
        $sectionName = $this->slug . '_section_serverpilot';
        $optionName = $this->option->getOptionName();
        $settings[$this->slug . '-serverpilot'] = [
			$sectionName,
			'Serverpilot Settings',
			[$this, 'serverpilot_settings'],
			$optionName,
            'settings' => [
            ]
        ];
        $settings[$this->slug . '-serverpilot']['settings'][] = [   
            $this->slug . '_field_platforms_serverpilot_enabled',
            'Serverpilot Enabled',
            [$this, 'plugin_setting_serverpilot_enabled'],
            $optionName,
            $sectionName,
            [   
                'label_for' => 'divimanager_options[serverpilot_enabled]'
            ]
        ];
        if ($this->serverpilot->isServerpilotEnabled()) {
            //['name' => 'ServerPilot API Url', 'type' => 'text', 'field' => 'spApiUrl', 'description' => 'ServerPilot API URL, should be set to https://api.serverpilot.io'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_api_url',
                'ServerPilot API Url',
                [$this, 'plugin_setting_api_url'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][api_url]'
                ],
                'ServerPilot API URL, should be set to https://api.serverpilot.io',
            ];
            //['name' => 'ServerPilot Client ID', 'type' => 'text', 'field' => 'spClientID', 'description' => 'Client ID for the ServerPilot account'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_client',
                'ServerPilot Client ID',
                [$this, 'plugin_setting_client'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][client]'
                ],
                'Client ID for the ServerPilot account',
            ];
            //['name' => 'ServerPilot API Key', 'type' => 'password', 'field' => 'spApiKey', 'description' => 'API Key for the client ID corredsponding to the ServerPilot account'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_api_key',
                'ServerPilot API Key',
                [$this, 'plugin_setting_api_key'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][api_key]'
                ],
                'API Key for the client ID corredsponding to the ServerPilot account',
            ];
            //['name' => 'ServerPilot Server ID', 'type' => 'text', 'field' => 'spServer', 'description' => 'ID corresponding to the server on Serverpilot to which new sites will be installed'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_server',
                'ServerPilot Server ID',
                [$this, 'plugin_setting_server'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][server]'
                ],
                'ID corresponding to the server on Serverpilot to which new sites will be installed',
            ];
            //['name' => 'ServerPilot SSH Key', 'type' => 'text', 'field' => 'spSshKey', 'description' => 'SSH Key to associate with the new site, if you have one'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_sshkey',
                'ServerPilot SSH Key',
                [$this, 'plugin_setting_sshkey'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][sshkey]'
                ],
                'SSH Key to associate with the new site, if you have one',
            ];
            // sp newsite runtime
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_runtime',
                'ServerPilot PHP Runtime',
                [$this, 'plugin_setting_runtime'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][runtime]'
                ],
                'PHP Runtime to use for the new site, should be php7.4',
            ];
            // sp newsite admin email
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_adminemail',
                'ServerPilot Admin Email',
                [$this, 'plugin_setting_adminemail'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][adminemail]'
                ],
                'Admin email for the new site',
            ];
            // sp newsite admin user
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_adminuser',
                'ServerPilot Admin User',
                [$this, 'plugin_setting_adminuser'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][adminuser]'
                ],
                'Admin username for the new site',
            ];

            //['name' => 'DME API Url', 'type' => 'text', 'field' => 'dmeApiUrl', 'description' => 'Dnsmadeeasy API URL, should be set to https://api.dnsmadeeasy.com/'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_dmeapiurl',
                'DME API Url',
                [$this, 'plugin_setting_dmeapiurl'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][dmeapiurl]'
                ],
                'Dnsmadeeasy API URL, should be set to https://api.dnsmadeeasy.com/',
            ];
            //['name' => 'DME API Key', 'type' => 'password', 'field' => 'dmeApiKey', 'description' => 'API key for dnsmadeeasy service'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_dmeapikey',
                'DME API Key',
                [$this, 'plugin_setting_dmeapikey'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][dmeapikey]'
                ],
                'API key for dnsmadeeasy service',
            ];
            //['name' => 'DME Secret Key', 'type' => 'password', 'field' => 'dmeSecretKey', 'description' => 'Secret key for dnsmadeeasy service'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_dmesecretkey',
                'DME Secret Key',
                [$this, 'plugin_setting_dmesecretkey'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][dmesecretkey]'
                ],
                'Secret key for dnsmadeeasy service',
            ];
            //['name' => 'DME Domain ID', 'type' => 'text', 'field' => 'dmeDomainID', 'description' => 'Domain ID on dnemadeeasy to which the staging domains will be added'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_dmedomainid',
                'DME Domain ID',
                [$this, 'plugin_setting_dmedomainid'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][dmedomainid]'
                ],
                'Domain ID on dnemadeeasy to which the staging domains will be added',
            ];
            //['name' => 'DME Server IP', 'type' => 'text', 'field' => 'dmeServerIP', 'description' => 'IP of the server on which the sites are being installed'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_dmeserverip',
                'DME Server IP',
                [$this, 'plugin_setting_dmeserverip'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][dmeserverip]'
                ],
                'IP of the server on which the sites are being installed',
            ];
            //['name' => 'Staging Domain', 'type' => 'text', 'field' => 'stagingDomain', 'description' => 'The domain the websites will be added to as subdomains'],
            $settings[$this->slug . '-serverpilot']['settings'][] = [
                $this->slug . '_field_platforms_serverpilot_dmestaging',
                'Staging Domain',
                [$this, 'plugin_setting_dmestaging'],
                $optionName,
                $sectionName,
                [   
                    'label_for' => 'divimanager_options[platforms][serverpilot][dmestaging]'
                ],
                'The domain the websites will be added to as subdomains',
            ];
        }

		return $settings;
    }

	public function admin_menu ()
    {
    }

    /**
     * custom plugin settings
     */
    public function serverpilot_settings($args)
    {   
        echo '<p id="' . esc_attr( $args['id'] ) . '">All the settings which the report aggregator extension will use.  Leave blank to use default values.</p>';
    }

    public function plugin_setting_serverpilot_enabled($args)
    {
        $enabled = $this->option->getOption('serverpilot_enabled');
        if (true == $enabled) {
            $checked = 'checked';
        } else {
            $checked = '';
        }
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="checkbox" size="40" value="true" '.$checked.'/>';
    }

    public function plugin_setting_api_url($args)
    {
        $api = $this->option->getOption('platforms')['serverpilot']['api_url'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $api ) . '" />';
    }

    public function plugin_setting_client($args)
    {
        $client = $this->option->getOption('platforms')['serverpilot']['client'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $client ) . '" />';
    }

    public function plugin_setting_api_key($args)
    {
        $key = $this->option->getOption('platforms')['serverpilot']['api_key'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="password" value="' . esc_attr ( $key ) . '" />';
    }

    public function plugin_setting_server($args)
    {
        $server = $this->option->getOption('platforms')['serverpilot']['server'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $server ) . '" />';
    }

    public function plugin_setting_sshkey($args)
    {
        $sshkey = $this->option->getOption('platforms')['serverpilot']['sshkey'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $sshkey ) . '" />';
    }

    public function plugin_setting_runtime($args)
    {
        $runtime = $this->option->getOption('platforms')['serverpilot']['runtime'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $runtime ) . '" />';
    }

    public function plugin_setting_adminemail($args)
    {
        $adminemail = $this->option->getOption('platforms')['serverpilot']['adminemail'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $adminemail ) . '" />';
    }

    public function plugin_setting_adminuser($args)
    {
        $adminuser = $this->option->getOption('platforms')['serverpilot']['adminuser'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $adminuser ) . '" />';
    }

    public function plugin_setting_dmeapiurl($args)
    {
        $dmeapiurl = $this->option->getOption('platforms')['serverpilot']['dmeapiurl'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $dmeapiurl ) . '" />';
    }

    public function plugin_setting_dmeapikey($args)
    {
        $dmeapikey = $this->option->getOption('platforms')['serverpilot']['dmeapikey'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="password" value="' . esc_attr ( $dmeapikey ) . '" />';
    }

    public function plugin_setting_dmesecretkey($args)
    {
        $dmesecretkey = $this->option->getOption('platforms')['serverpilot']['dmesecretkey'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="password" value="' . esc_attr ( $dmesecretkey ) . '" />';
    }

    public function plugin_setting_dmedomainid($args)
    {
        $dmedomainid = $this->option->getOption('platforms')['serverpilot']['dmedomainid'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $dmedomainid ) . '" />';
    }

    public function plugin_setting_dmeserverip($args)
    {
        $dmeserverip = $this->option->getOption('platforms')['serverpilot']['dmeserverip'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $dmeserverip ) . '" />';
    }

    public function plugin_setting_dmestaging($args)
    {
        $dmestaging = $this->option->getOption('platforms')['serverpilot']['dmestaging'];
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $dmestaging ) . '" />';
    }

    public function plugin_options_validate($input)
    {
        $input['platforms']['serverpilot']['enabled'] = isset($input['platforms']['serverpilot']['enabled']) ? true == \sanitize_text_field($input['platforms']['serverpilot']['enabled']) : false;
        $input['platforms']['serverpilot']['api_url'] = isset($input['platforms']['serverpilot']['api_url']) ? \sanitize_text_field($input['platforms']['serverpilot']['api_url']) : null;
        $input['platforms']['serverpilot']['client'] = isset($input['platforms']['serverpilot']['client']) ? \sanitize_text_field($input['platforms']['serverpilot']['client']) : null;
        $input['platforms']['serverpilot']['api_key'] = isset($input['platforms']['serverpilot']['api_key']) ? \sanitize_text_field($input['platforms']['serverpilot']['api_key']) : null;
        $input['platforms']['serverpilot']['server'] = isset($input['platforms']['serverpilot']['server']) ? \sanitize_text_field($input['platforms']['serverpilot']['server']) : null;
        $input['platforms']['serverpilot']['sshkey'] = isset($input['platforms']['serverpilot']['sshkey']) ? \sanitize_text_field($input['platforms']['serverpilot']['sshkey']) : null;
        $input['platforms']['serverpilot']['runtime'] = isset($input['platforms']['serverpilot']['runtime']) ? \sanitize_text_field($input['platforms']['serverpilot']['runtime']) : null;
        $input['platforms']['serverpilot']['adminemail'] = isset($input['platforms']['serverpilot']['adminemail']) ? \sanitize_text_field($input['platforms']['serverpilot']['adminemail']) : null;
        $input['platforms']['serverpilot']['adminuser'] = isset($input['platforms']['serverpilot']['adminuser']) ? \sanitize_text_field($input['platforms']['serverpilot']['adminuser']) : null;

        $input['platforms']['serverpilot']['dmeapiurl'] = isset($input['platforms']['serverpilot']['dmeapiurl']) ? \sanitize_text_field($input['platforms']['serverpilot']['dmeapiurl']) : null;
        $input['platforms']['serverpilot']['dmeapikey'] = isset($input['platforms']['serverpilot']['dmeapikey']) ? \sanitize_text_field($input['platforms']['serverpilot']['dmeapikey']) : null;
        $input['platforms']['serverpilot']['dmesecretkey'] = isset($input['platforms']['serverpilot']['dmesecretkey']) ? \sanitize_text_field($input['platforms']['serverpilot']['dmesecretkey']) : null;
        $input['platforms']['serverpilot']['dmedomainid'] = isset($input['platforms']['serverpilot']['dmedomainid']) ? \sanitize_text_field($input['platforms']['serverpilot']['dmedomainid']) : null;
        $input['platforms']['serverpilot']['dmeserverip'] = isset($input['platforms']['serverpilot']['dmeserverip']) ? \sanitize_text_field($input['platforms']['serverpilot']['dmeserverip']) : null;
        $input['platforms']['serverpilot']['dmestaging'] = isset($input['platforms']['serverpilot']['dmestaging']) ? \sanitize_text_field($input['platforms']['serverpilot']['dmestaging']) : null;

        return $input;
    }
}
