<?php declare(strict_types=1);
namespace DiviManagerPro\Service;

use DiviManager\Service\DbManager;
use DiviManagerPro\Model\Server as ServerModel;
use DiviManagerPro\Platform\PlatformInterface as PlatformModel;
use DiviManagerPro\Service\Server as ServerService;
use DiviManagerPro\Service\Platform as PlatformService;

/**
 * Service class for working with platform server joins
 */
class PlatformServer extends DbManager
{
    public function __construct()
    {
        $fields =<<<EOSQL
platform_id varchar(64) null,
server_id int NOT NULL
EOSQL;

        parent::__construct('divimanager_pro_platforms_servers', $fields);
    }

    /**
     * Set the platform service
     */
    public function setPlatformService(PlatformService $p)
    {
        $this->platformService = $p;
    }

    /**
     * Set the server service
     */
    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    /**
     * Get servers for given platform
     *
     * @param Platform
     * @return array
     */
    public function getServersForPlatform( PlatformModel $platform )
    {
        $ids = $this->getFieldByField('server_id', ['platform_id' => $platform->getID()]);

        if (null === $ids) {
            return [];
        }

        return $this->serverService->getServersByID($ids);
    }

    /**
     * Get servers for multiple platforms
     *
     * @param Platform
     * @return array
     */
    public function getServersForPlatforms( array $platforms = [], $options = null )
    {
        if (empty($platforms)) {
            return [];
        }

        $platformIDs = [];
        foreach ($platforms as $platform) {
            $platformIDs[] = $platform->getId();
        }

        $ids = $this->getFieldByField('server_id', ['platform_id' => $platformIDs]);

        if (null === $ids) {
            return [];
        }

        return $this->serverService->getServersByID($ids);
    }

    /**
     * Add server to platform
     *
     * @param ServerModel
     * @param PlatformModel
     */
    public function addServerToPlatform( ServerModel $server, PlatformModel $platform)
    {
        $this->wpdb->insert(
            $this->getTableName(),
            [
                'platform_id' => $platform->getId(),
                'server_id' => $server->getId(),
            ],
            [
                '%s',
                '%d',
            ]
        );
    }

    /**
     * Remove server from platform
     *
     * @param ServerModel
     * @param PlatformModel
     */
    public function removeServerFromPlatform( ServerModel $server, PlatformModel $platform)
    {
        $this->wpdb->delete(
            $this->tableName,
            [
                'platform_id' => $platform->getId(),
                'server_id' => $server->getId(),
            ],
            [
                '%s',
                '%d',
            ]
        );
    }
}
