<?php declare(strict_types=1);
namespace DiviManagerPro\Service;

use DiviManagerPro\Model\Server as ServerModel;
use DiviManager\Service\DbManager;

/**
 * Service class for working with servers
 *
 * Handles database operations: insert, update, delete, select
 */
class Server extends DbManager
{
    public function __construct()
    {
        $fields =<<<EOSQL
id int NOT NULL AUTO_INCREMENT,
name varchar(255) NOT NULL,
ip varchar(255) NOT NULL,
ipv6 varchar(255) NULL,
serverID varchar(255) NOT NULL,
PRIMARY KEY (id)
EOSQL;

        parent::__construct('divimanager_pro_servers', $fields);
    }

    public function newModel ()
    {
        return new ServerModel;
    }

    public function getServers ($perPage = 150, $pageNumber = 1, $options = null)
    {
        return $this->getRecords($perPage, $pageNumber, $options);
    }

    public function getServer (int $id)
    {
        return $this->getRecord($id);
    }

    public function getServersByID (array $ids)
    {
        return $this->getRecordsByID($ids);
    }

    public function getServerByName ($name)
    {
        return $this->getRecordByField('name', $name, '%s');
    }

    public function createServer (ServerModel $server)
    {
        $ok = $this->wpdb->insert( $this->getTableName(), [
            'name' => $server->getName(),
            'ip'   => $server->getIp(),
            'ipv6' => $server->getIpv6(),
            'serverID' => $server->getServerID(),
        ]);

        if (false === $ok) {
            throw new \Exception('Failed to create server:' . print_r($server->getId(), true) . ':' . print_r($server, true));
        }

        $server->setId($this->wpdb->insert_id);

        return $server;
    }

    public function updateServer (ServerModel $server)
    {
        $result = $this->wpdb->update( $this->getTableName(), [
            'name' => $server->getName(),
            'ip'   => $server->getIp(),
            'ipv6' => $server->getIpv6(),
            'serverID' => $server->getServerID(),
        ], ['id' => $server->getId()]);

        if (false === $result) {
            throw new \Exception('Failed to update server:' . print_r($server->getId(), true) . ':' . print_r($server, true));
        }

        return $server;
    }

    public function deleteServer (ServerModel $server)
    {
        return $this->deleteRecord($server);
    }
}
