<?php declare(strict_types=1);
namespace DiviManagerPro\Woocommerce\Table;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Manager\Site as SiteManager;
use DiviManager\Manager\Queue as QueueManager;
use DiviManager\Manager\Profile as ProfileManager;
use DiviManager\Manager\Theme as ThemeManager;
use DiviManager\Manager\Plugin as PluginManager;
use DiviManager\Manager\Worker as WorkerManager;
use DiviManager\Action\Site as SiteAction;
use DiviManager\Model\Site as SiteModel;

use DiviManager\Helper;

use DiviManagerPro\Woocommerce\Woocommerce as DMPWC;
use DiviManager\Service\Plugin as PluginService;
use DiviManager\Service\Site as SiteService;

/**
 * Extend WP_List_Table
 *
 * @see https://www.sitepoint.com/using-wp_list_table-to-create-wordpress-admin-tables/
 */
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

/**
 * List all the orders in the store
 */
class Order extends \WP_List_Table
{
    public $stats;

    /**
     * @var DMPWC
     */
    protected $dmpwc;

    /**
     * @var PluginService
     */
    protected $pluginService;

    /**
     * @var SiteService
     */
    protected $siteService;

	public function __construct()
	{
		parent::__construct([
			'singular' => __( 'Order' ),
			'plural'   => __( 'Orders' ),
            'ajax'     => false,
            'screen'   => 'woocommerceordertable',
        ]);

        $this->stats = [
            'total' => 0,
        ];

	}

    public function setDmpwc(DMPWC $dmpwc)
    {
        $this->dmpwc = $dmpwc;
    }

    public function setPluginService(PluginService $pluginService)
    {
        $this->pluginService = $pluginService;
    }

    public function setSiteService(SiteService $siteService)
    {
        $this->siteService = $siteService;
    }

    public function no_items()
    {
        _e( 'No orders found in the store' );
    }

    /**
     * Columns to display
     *
     * @return array
     */
    public function get_columns()
    {
        $columns = [
            'cb'      => '<input type="checkbox" />',
            'number'  => __( 'Order #' ),
            'domain'  => __( 'Domain' ),
            'profile' => __( 'Profile' ),
            'site'    => __( 'Site' ),
        ];

        return $columns;
    }

	/**
	 * Sortable columns
	 *
	 * @return array
	 */
    public function get_sortable_columns()
    {
		$columns = [
			//'number' => [ 'number', false ],
        ];

		return $columns;
    }

    /**
     * Render the bulk action checkbox
     *
     * @param array $order
     *
     * @return string
     */
    function column_cb( $order ) {
        return sprintf(
            '<input type="checkbox" name="bulk-action[]" value="%s" />', $order['order']
        );
    }

    /**
     * Default render of column value
     *
     * @param array $order
     * @param string $column
     *
     * @return mixed
     */
    public function column_default( $order, $column )
    {
        switch ( $column ) {
        case 'domain':
            return sprintf('<span class="">%s</span>', $order['domain']);

        case 'profile':
            return sprintf('<span class="">%s</span>', $order['profile']->getName());

        default:
            // an unmatched column, show whole row
            return print_r( $plugin, true );
        }
    }

    /**
     * Provide the order column
     *
     * @param array $order
     *
     * @return string
     */
    public function column_number( $order )
    {
        $bulk_nonce = wp_create_nonce( 'sl_manager_bulk_action' );

        $number = sprintf('<span class="">%s</span>', $order['order']);

        return $number;
        //$id = $plugin->getId();
        //$detailUrl = esc_url ( add_query_arg( ['page' => 'slmanager-plugin-view', 'plugin_id' => $id], admin_url('admin.php')));
        //$editUrl = esc_url ( add_query_arg( ['page' => 'slmanager-plugin-update', 'plugin_id' => $id], admin_url('admin.php')));
        //$deleteUrl = esc_url ( add_query_arg( ['page' => 'slmanager-plugin-delete', 'plugin_id' => $id], admin_url('admin.php')));
        //$actions = [
            //'detail' => sprintf( '<a href="%s">Details</a>', $detailUrl),
            //'edit' => sprintf( '<a href="%s">Edit</a>', $editUrl),
            //'delete' => sprintf( '<a href="%s" onclick="return confirm(\'Press OK to delete this plugin\');">Delete</a>', $deleteUrl),
        //];

        //return $name . $this->row_actions( $actions );
    }

    /**
     * Provide the site column
     *
     * @param array $order
     *
     * @return string
     */
    public function column_site( $order )
    {
        $bulk_nonce = wp_create_nonce( 'sl_manager_bulk_action' );

        if (! isset($order['site']) || null === $order['site']) {
            $site = 'N/A';
        } else {
            $site = sprintf('<span class="">%s</span>', $order['site']->getName());
            //$site = sprintf('<span class="">%s</span>', 'Registered');
        }

        return $site;
        //$id = $plugin->getId();
        //$detailUrl = esc_url ( add_query_arg( ['page' => 'slmanager-plugin-view', 'plugin_id' => $id], admin_url('admin.php')));
        //$editUrl = esc_url ( add_query_arg( ['page' => 'slmanager-plugin-update', 'plugin_id' => $id], admin_url('admin.php')));
        //$deleteUrl = esc_url ( add_query_arg( ['page' => 'slmanager-plugin-delete', 'plugin_id' => $id], admin_url('admin.php')));
        //$actions = [
            //'detail' => sprintf( '<a href="%s">Details</a>', $detailUrl),
            //'edit' => sprintf( '<a href="%s">Edit</a>', $editUrl),
            //'delete' => sprintf( '<a href="%s" onclick="return confirm(\'Press OK to delete this plugin\');">Delete</a>', $deleteUrl),
        //];

        //return $name . $this->row_actions( $actions );
    }

	/**
	 * Define the bulk actions this table supports
	 *
	 * @return array
	 */
	public function get_bulk_actions() {
	    $actions = [
		    'bulk-import'   => 'Import Orders',
	    ];

	    return $actions;
	}

    /**
     * Prepare the data for the table
     */
    public function prepare_items()
    {
        $this->_column_headers = $this->get_column_info();
        $this->process_bulk_action();

        var_dump(__FUNCTION__);
        $this->items = [];
        $this->stats['total'] = 0;
        return;

        //$perPageOption = get_current_screen()->get_option('per_page');
        ////$perPage    = $this->get_items_per_page( 'plugins_per_page' , $perPageOption['default'] );
        //$pageNumber = $this->get_pagenum();

        $orders = $this->dmpwc->getProfileSubscriptions();
        foreach ($orders as $key => $order) {
            $site = $this->siteService->getSiteByDomain($order['domain']);
            $orders[$key]['site'] = $site;
        }
        $count = count($orders);

        $this->set_pagination_args( [
            'total_items' => $count,
            //'per_page'    => $perPage
        ] );

        $this->items = $orders;
        $this->stats['total'] = $count;
    }

    public function check_nonce()
    {
        $nonce = esc_attr( $_REQUEST['_wpnonce'] );
        $result =  wp_verify_nonce( $nonce, 'sl_manager_bulk_action' );
        if ( ! $result) {
            wp_die('error with nonce');
        }

        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }
    }
    /**
     * Process bulk and single action links
     */
    public function process_bulk_action()
    {
        if (empty($_POST) && ! empty($this->current_action())) {
            $this->check_nonce();
            $plugin = isset($_GET['id']) ? $this->pluginService->getPlugin(sanitize_text_field($_GET['id'])) : null;

        } else {
            // forms have their own nonce processing
            if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-import' )
                || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-import' )
                ) {

                $ids = esc_sql( $_POST['bulk-action'] );
                //$orders = $dmpwc->getProfileOrders();
                $orders = $this->dmpwc->getProfileSubscriptions();

                Helper::adminMessage('Importing selected orders', Helper::NOTICE);
                foreach ($ids as $id) {
                    $found = false;
                    foreach($orders as $order) {
                        if ($id == $order['order']) {
                            $found = true;
                            break; // found the order
                        }
                    }
                    if (false === $found) {
                        // order not found for importing
                        // how?
                        Helper::adminMessage('Order not found for import: ' . $id, Helper::ERROR);
                        break; // not found somehow, skip it
                    }
                    $site = $this->siteService->getSiteByDomain($order['domain']);
                    if ($site) {
                        Helper::adminMessage('Site found, updating for order: ' . $id, Helper::NOTICE);

                        // site already exists
                        // maybe update the order and customer?
                        $site->setUserID( $order['customer'] );
                        $site->setOrderID( $order['order'] );
                        $site->setProfileID( $order['profile']->getId() );
                        $site->setTimestamp(null);
                        $this->siteService->updateSite($site);
                    } else {
                        Helper::adminMessage('Site not found, creating new site for order: ' . $id, Helper::NOTICE);
                        $site = $this->siteService->newModel();
                        $site->setUrl( $order['domain'] );
                        $site->setDomain( $order['domain'] );
                        $site->setName( $order['domain'] );
                        $site->setUserID( $order['customer'] );
                        $site->setOrderID( $order['order'] );
                        $site->setProfileID( $order['profile']->getId() );
                        $site->setTimestamp(null);
                        $this->siteService->createSite($site);
                    }
                }
            }
        }
    }

    public function getStats()
    {
        return $this->stats;
    }
}
