<?php

namespace DNSMadeEasyAPI;

use GuzzleHttp\Client;

class DNSMadeEasyAPI
{
    /**
     * @var String Base site url for api
     */
    protected $siteUrl;

    /**
     * @var String authentication api key
     */
    protected $apiKey;

    /**
     * @var String authentication secret key
     */
    protected $secretKey;

    /**
     * @var Client our GuzzleHttp/Client instance
     */
    protected $guzzle;

    /**
     * Constructor for creating a new DNSMadeEasy API connection instance
     */
    public function __construct($siteUrl, $apiKey, $secretKey)
    {
        $this->siteUrl = $siteUrl;
        $this->apiKey = $apiKey;
        $this->secretKey = $secretKey;
        $this->guzzle = null;

        $this->start();
    }

    public function setToken($token)
    {
        $this->token = $token;
        $this->guzzle->setHeader('Authorization', 'Bearer ' . $this->token);
    }

    protected function getRecordRequest()
    {
        $req = new Request\Record($this->guzzle, $this->siteUrl);
        $req->setApiKey($this->apiKey);
        $req->setSecretKey($this->secretKey);

        return $req;
    }

    public function start()
    {
        $this->guzzle = new Client(['base_uri' => $this->siteUrl]);
    }

    /**
     * Close not supported in dnsmadeeasy api
     */
    public function close()
    {
    }

    /**
     * Login not supported in dnsmadeeasy api
     */
    public function login()
    {
    }

    /**
     * Logout not supported in dnsmadeeasy api
     */
    public function logout()
    {
    }

    /**
     * Get dns records for a domain
     */
    public function getRecords($domain)
    {
        $records = $this->getRecordRequest()->records($domain);
        return $records->data;
    }

    /**
     * Add a dns record for a domain
     */
    public function addRecord($name, $domain, $ip)
    {
        $record = $this->getRecordRequest();
        return $record->create($name, $domain, $ip);
    }

    public function getGuzzle()
    {
        return $this->guzzle;
    }
}
