<?php

namespace DNSMadeEasyAPI\Request;

use Curl\Curl;

class Auth extends AbstractRequest
{
    /**
     * Login to the dnsmadeeasy api
     *
     * @param email String The email address to login as
     * @param credential String The credential for this account
     *
     * @return token String The authentication token for use with future requests
     */
    public function login($secret, $credential)
    {
        $format = 'D, d M Y H:i:s \G\M\T';
        $now = new \DateTime(null, new \DateTimeZone('UTC'));
        $nowFormatted = $now->format($format);
        $hmac = \hash_hmac('sha1', $nowFormatted, $secret, true);

        $authUrl = $this->siteUrl . '/auth/token';

        $this->curl->post($authUrl, [
            'email' => $email,
            'password' => $credential,
            'grant_type' => 'password'
        ]);

        if ($this->curl->error) {
            throw new \RuntimeException('Error: ' . $this->curl->errorCode . ': ' . $this->curl->errorMessage);
        }

        $token = $this->curl->response->access_token;

        return $token;
    }

    public function refresh()
    {
        $authUrl = $this->siteUrl . '/auth/refresh';

        $this->curl->post($authUrl, [
            'grant_type' => 'refresh_token',
            'refresh_token' => $this->token
        ]);
    }

    public function revoke()
    {
        $authUrl = $this->siteUrl . '/auth/revoke';

        $this->curl->post($authUrl, [
            'token' => $this->token
        ]);
    }
}
