<?php

namespace DNSMadeEasyAPI\Request;

use GuzzleHttp\Client;

// wrapper for the Record API endpoint
class Record extends AbstractRequest
{
    public $endpoint = '/V2.0/dns/managed';

    public function get($domain)
    {
        $format = 'D, d M Y H:i:s \G\M\T';
        $format = 'D, d M Y H:i:s e';
        $now = new \DateTime('now', new \DateTimeZone('GMT'));
        $nowFormatted = $now->format($format);
        $hmac = \hash_hmac('sha1', $nowFormatted, $this->secretKey);

        $response = $this->client->get($this->endpoint . '/' . $domain, [
            'headers' => [
                'x-dnsme-apiKey' => $this->apiKey,
                'x-dnsme-requestDate' => $nowFormatted,
                'x-dnsme-hmac' => $hmac
            ]
        ]);

        return \json_decode($response->getBody());
    }

    public function records($domain)
    {
        $format = 'D, d M Y H:i:s \G\M\T';
        $format = 'D, d M Y H:i:s e';
        $now = new \DateTime('now', new \DateTimeZone('GMT'));
        $nowFormatted = $now->format($format);
        $hmac = \hash_hmac('sha1', $nowFormatted, $this->secretKey);

        $response = $this->client->get($this->endpoint . '/' . $domain . '/records', [
            'headers' => [
                'x-dnsme-apiKey' => $this->apiKey,
                'x-dnsme-requestDate' => $nowFormatted,
                'x-dnsme-hmac' => $hmac
            ]
        ]);

        return \json_decode($response->getBody());
    }

    public function create($name, $domain, $ip)
    {
        $format = 'D, d M Y H:i:s \G\M\T';
        $format = 'D, d M Y H:i:s e';
        $now = new \DateTime('now', new \DateTimeZone('GMT'));
        $nowFormatted = $now->format($format);
        $hmac = \hash_hmac('sha1', $nowFormatted, $this->secretKey);

        $response = $this->client->post($this->endpoint . '/' . $domain . '/records', [
            'headers' => [
                'x-dnsme-apiKey' => $this->apiKey,
                'x-dnsme-requestDate' => $nowFormatted,
                'x-dnsme-hmac' => $hmac
            ],
            'json' => [
                    'name' => $name,
                'type' => 'A',
                'value' => $ip,
                'gtdLocation' => 'DEFAULT',
                'ttl' => 1800
            ],
        ]);

        return \json_decode($response->getBody());
    }
}
