<?php

declare(strict_types=1);

namespace ServerPilotAPI\Request;

use PHPUnit\Framework\TestCase;
use GuzzleHttp\Client;

final class AbstractRequestTest extends TestCase
{
    public function testConstructorRequiresGuzzleClient(): void
    {
        $this->expectException(\TypeError::class);
        $sut = new Site('not a guzzle client', 'foo');
    }

    public function testConstructor()
    {
        $client = new Client();
        $url = 'foo';
        $sut = new Site($client, $url);

        $this->assertEquals(
            $client,
            $sut->getClient()
        );
        $this->assertEquals(
            $url,
            $sut->getSiteUrl()
        );
    }

    public function testClientIDAccessor()
    {
        $clientID = 'clientID';
        $sut = new Site(new Client(), 'foo');
        $sut->setClientID($clientID);
        $this->assertEquals(
            $clientID,
            $sut->getClientID()
        );
    }

    public function testApiKeyAccessor()
    {
        $apiKey = 'apiKey';
        $sut = new Site(new Client(), 'foo');
        $sut->setApiKey($apiKey);
        $this->assertEquals(
            $apiKey,
            $sut->getApiKey()
        );
    }
}
