<?php

declare(strict_types=1);

namespace ServerPilotAPI\Request;

use PHPUnit\Framework\TestCase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Exception\RequestException;

final class ActionTest extends TestCase
{
    public function testActionOpen(): void
    {
        $responseData = [
            'data' =>  ['status' => 'open'],
        ];
        $response = json_encode($responseData);
        $mock = new MockHandler([
            new Response(200, [], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Action($client, $url);
        $response = $sut->get(1);
        $this->assertEquals(
            Action::OPEN,
            $response
        );
    }
    public function testActionUnknown(): void
    {
        $responseData = [
            'data' =>  ['status' => 'anything'],
        ];
        $response = json_encode($responseData);
        $mock = new MockHandler([
            new Response(200, [], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Action($client, $url);
        $response = $sut->get(1);
        $this->assertEquals(
            Action::ERROR,
            $response
        );
    }
    public function testActionFailed(): void
    {
        $responseData = [
            'data' =>  ['status' => 'error'],
        ];
        $response = json_encode($responseData);
        $mock = new MockHandler([
            new Response(200, [], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Action($client, $url);
        $response = $sut->get(1);
        $this->assertEquals(
            Action::ERROR,
            $response
        );
    }
    public function testActionSuccess(): void
    {
        $responseData = [
            'data' =>  ['status' => 'success'],
        ];
        $response = json_encode($responseData);
        $mock = new MockHandler([
            new Response(200, [], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Action($client, $url);
        $response = $sut->get(1);
        $this->assertEquals(
            Action::SUCCESS,
            $response
        );
    }
}
