<?php

declare(strict_types=1);

namespace ServerPilotAPI;

use PHPUnit\Framework\TestCase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Exception\RequestException;

final class ServerPilotAPITest extends TestCase
{
    public function testDefaultGuzzleConfig(): void
    {
        $sut = new ServerPilotAPI('foo', 'bar', 'baz');
        $client = $sut->getGuzzle();
        $this->assertInstanceOf(
            Client::class,
            $client
        );
        $this->assertEquals(
            'foo',
            $client->getConfig('base_uri')
        );
    }

    public function testAddDomainToSite(): void
    {
        $expected = (object) [
            'actionid' => 'KlsNzLikw3BRvShc',
            'data' => (object) [
                'id' => 'nlcN0TwdZAyNEgdp',
                'datecreated' => 1403143012,
                'name' => 'gallery',
                'sysuserid' => 'RvnwAIfuENyjUVnl',
                'domains' => ['foo.bar'],
                'ssl' => null,
                'serverid' => '4zGDDO2xg30yEeum',
                'runtime' => 'php5.6'
            ]
        ];
        $response = json_encode($expected);
        $mock = new MockHandler([
            new Response(200, ['X-Foo' => 'Bar'], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new ServerPilotAPI('url', 'clientid', 'apikey');
        $domains = ['foo.bar'];
        //$response = $sut->addDomainToSite(1, 'bar.baz');
        //$this->assertEquals(
            //$expected,
            //$response
        //);
    }
}
